var choose = "Choose Problem";
var problems = [new FarmerProblem(), new WaterJugProblem()];
var problem = problems[0];

// Variables representing jQuery objects
var problemPanel;
var selector;
var probName;
var intro;
var state;
var moveList;
var feedback;
var resetButton;

var bSize;

var delay = 500;

function init() { // called when the document is ready
    
    problemPanel = $("#problemPanel");
    moveList = $("#moveTable");
    state = $("#state");
    probName = $("#probName");
    intro = $("#intro");
    feedback = $("#message");
    
    resetButton = $("#reset");
    resetButton.click(function() { 
        reset(); 
    });
    
    selector = $("#selector");
    
    selector.append($("<option></option>").text(choose));  // Default selector option
    
    selector.change(function() {                           
        if (selector.val() === choose) {
            problemPanel.slideUp(delay);
        }
        else {
            problemPanel.slideUp(delay, function(){   // Note use of callback
                changeProblem();                      // function to avoid
                problemPanel.slideDown(delay);        // timing issues
            });
        }
    });
    
    problems.forEach(function(p) {
        selector.append($("<option></option>").text(p.name));
    });
    
    problemPanel.hide();   // Do not show problem panel initially
}

function changeProblem() {
    
    problem = problemForName(selector.val());
    probName.text(problem.name);
    intro.text(problem.introduction);
    state.attr("cols", "" + problem.currentState.width);
    state.attr("rows", "" + problem.currentState.height);
    updateState();
    clearTable();
    clearMessage();
    bSize = computeButtonSize();
    
    problem.moveNames.forEach(function(move) {
        var item = $("<li></li>").append(createButton(move));
        moveList.append(item); 
    });
}

function problemForName(probName) {    
    var prob;
    problems.forEach(function(p) {
        if (p.name === probName) prob = p;
    });  
    return prob;
}

function clearTable() {
    moveList.empty();
}

function computeButtonSize() { // determines largest move name so move buttons
    var size = 0;              // have uniform size
    problem.moveNames.forEach(function(move) {
        if (move.length > size) {
            size = move.length;
        }
    });
    return size + 1;    // add some length to provide a margin
}

function createButton(move) {
    var button = $("<input></input>");
    button.attr("type", "button");
    button.val(move);
    button.css("width", bSize + "ex");
    button.click(function() {
        clearMessage();
        tryMove(move);
    });
    return button;
}

function tryMove(move) {
    var newState = problem.mover.doMove(move, problem.currentState);
    if (newState === null)
        displayMessage("That move is not possible");
    else {
        problem.currentState = newState;
        updateState();
    }
    if (problem.success()) {
        displayMessage("Congratulations. You solved the problem.");
    }
}

function displayMessage(message) {
    feedback.text(message);
}

function clearMessage() {
    displayMessage("");
}

function updateState() {
    state.val(problem.currentState);
}

function reset() {
    problem.currentState = problem.initialState;
    updateState();
    clearMessage();
}
